/********************************************************************************/
/*																				*/
/*	IMG_DISPLAY.CPP																*/
/* 																				*/
/*	(c) 2000 Hunt Engineering													*/
/*	    All Rights Reserved														*/
/*																				*/
/*  This file contains the PC code for the	image display demo.					*/
/* 																				*/
/* 	Version		Date		Version Author	Comments							*/
/*	-------		----		--------------	--------							*/
/* 																				*/
/*		1.0		10-10-03	PW			    Imaging with FPGA demo          	*/
/*		1.1		11-05-04	JT				Character array sizes now all 1024	*/
/*											Added HeReset at start of Thread	*/
/*		1.2		03-08-04	JT				Changed 'exec' calls to hrn_fpga	*/
/*											and HeartConf to library calls.		*/
/*											Changed thread a little so that		*/
/*											upon error the thread restarts.		*/
/*		1.3		01-10-04	RW				Modified HSB_Setup function.		*/
/*											Some cameras present an incomplete  */
/*											number of pixels in the very first  */
/*											line. For safety, the region of     */
/*											interest should begin with line 1   */
/*											rather than line 0.					*/
/* 																				*/
/********************************************************************************/

#include "stdafx.h"


#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <conio.h>
#include <time.h>
#include <process.h>

#include "resource.h"

/********************************************/
/*	Header Files for DSP Board etc			*/
/********************************************/
#include "heapi.h"
#include "hrn_fpga\inc\hrn_fpga.h"
#include "hc.h"
#include "image_defs.h"
#include "msg_defs.h"
#include "demo_defs.h"
#include "globals.h"

/****************************************************/
/* Definitions for HE API							*/
/****************************************************/
HE_HANDLE 		hDevice=NULL;
HE_HANDLE 		hReset=NULL;
HE_IOSTATUS 	ReadIoStatus=NULL;

/****************************************************/
/* Function / Variable Definitions					*/
/****************************************************/
LRESULT CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM) ;
HBITMAP CreateDibSectionFromDibFile (PTSTR szFileName);
TCHAR szAppName[] = TEXT ("Img_Display") ;


typedef struct
{
	int cam_control;
	int cam_use_dval;
	int cam_polarity;
	int visible;
} CAMERA;

CAMERA d;




int   Initialise_Host_HSB(char *dev_name, HE_WORD board);
void  Print_Win32_Error(HE_DWORD errorcode);
int  error(char *string, int Status);
void  Display_Help(void);
void SendUserData(char *devname, HE_WORD Board, HE_BYTE slot, HE_BYTE Address, int Data);
void Error(HE_DWORD errorcode);
HANDLE		hThread;

int readback_user(HE_BYTE slot, HE_BYTE *reply, int buffer_size, HE_BYTE send_data);
void writeReadCheck(int m, HE_DWORD *pattern, HE_DWORD *results, HE_DWORD *Exp_Ans);
long WINAPI Thread(PVOID pvoid);
VOID CALLBACK Statsfunc(HWND hwnd, UINT uint, UINT uint2, ULONG longd);
int Initialise_Host_HSB(char *dev_name, HE_WORD board);
void SendUserData(char *devname, HE_WORD Board, HE_BYTE slot, HE_BYTE Address, int Data);
int readback_user(HE_BYTE slot, HE_BYTE *reply, int buffer_size, HE_BYTE user_addr);
HE_BYTE readUserData(HE_BYTE slot, HE_BYTE *reply, int buffer_size, HE_BYTE user_addr);
void setConvolutionConstants(char *devname, HE_WORD Board, int Slot, unsigned long command);
int hsb_setup(char *devname, HE_WORD Board, HE_BYTE Slot,CAMERA *d);

// Global Variables:
HINSTANCE hInst;								// current instance

/****************************************************/
/* Thread management structure						*/
/****************************************************/
typedef struct
{
	HWND		hwnd;
	BOOL		bContinue;
}
PARAMS, *PPARAMS;
static	PARAMS params;
unsigned long DSP_ThreadID;
unsigned long Stats_ThreadID;
volatile PPARAMS pparams;
int runstats;


/****************************************************/
/* List pointers for two bitmaps					*/
/****************************************************/
HE_DWORD 		Status;

HE_WORD Board=0; // The board number we will be using
HE_HANDLE   hsbDevice		= NULL;
BYTE		*pimage1_bits;	// pointer to image 1 pixels
BYTE		*pimage2_bits;	// pointer to image 2 pixels
BYTE		*currentBitMap;	// pointer to current Image pixels
HBITMAP		hBitmap1 ;		// handle for bitmap 1
HBITMAP		hBitmap2 ;		// handle for bitmap 2
HBITMAP		hBitmap_Disp ;	// handle for display bitmap
BITMAPINFOHEADER bmih;
RGBQUAD		ColorTable[256];
HDC			hdcMem ;
long n=0;

// Server / Loader stuff
int SL_Status=0;
int		Status_Changed = TRUE;
char	Status_Text[256];
unsigned int Screen_Draw = TRUE;
int verbose;
char devname[1024] = "hep9a";
int demo_mode = NONE;
int demo_runmode = 0;
int goodframes =0;
TCHAR *comment_pointer = START_COMMENT;

/****************************************************/

char mybitstream[1024];
char exepath[1024];

int    modsdram = 0;
int    slotnumber=1;
HANDLE startsem;	/* exclusive access for printf etc */


int browseforbitstream(HWND hDlg)
{
	CFileDialog browse(TRUE);
	char *fname, str[1024];

	strcpy(str, "..\\bitstreams");

	browse.m_ofn.lpstrInitialDir =  str;
	browse.m_ofn.lpstrFilter     = "RBT/HCB File (*.rbt,*.hcb)\0*.rbt;*.hcb\0RBT File (*.rbt)\0*.rbt\0HCB File (*.hcb)\0*.hcb\0All Files (*.*)\0*.*\0\0\0";

	switch (browse.DoModal())
	{
	case IDOK:		fname = browse.m_ofn.lpstrFile;
					strcpy(mybitstream, fname);
					return 1;
	case IDCANCEL:	return 0;
	default:		return 0;
	}
}

int UpdateFileMenu(HWND hWnd, char *bitstream)
{
	HMENU hMenu, sMenu;

	hMenu = GetMenu(hWnd);
	if (hMenu==NULL)
	{
		MessageBox(hWnd, "main", "Registry sub-menu", MB_OK);
		return 0;
	}
	sMenu = GetSubMenu(hMenu, 0);	/* File */
	if (sMenu==NULL)
	{
		MessageBox(hWnd, "tools", "Registry sub-menu", MB_OK);
		return 0;
	}

	if (DeleteMenu(sMenu, IDD_BITSTREAM, MF_BYCOMMAND))
	{
		AppendMenu(sMenu, MF_ENABLED, IDD_BITSTREAM, bitstream);
	}
	if (DeleteMenu(sMenu, IDD_SLOT, MF_BYCOMMAND))
	{
		AppendMenu(sMenu, MF_ENABLED, IDD_SLOT, "&slotnumber");
	}
	if (DeleteMenu(sMenu, IDD_SDRAMSELECT, MF_BYCOMMAND))
	{
		AppendMenu(sMenu, MF_ENABLED, IDD_SDRAMSELECT, "SDRAM");
		if (modsdram) CheckMenuItem(sMenu, IDD_SDRAMSELECT, MF_BYCOMMAND | MF_CHECKED);
	}
	if (DeleteMenu(sMenu, ID_DEMO_SELFRUNNING, MF_BYCOMMAND))
	{
		AppendMenu(sMenu, MF_ENABLED, ID_DEMO_SELFRUNNING, "Self Running");
			if(demo_runmode==0)	CheckMenuItem(sMenu, ID_DEMO_SELFRUNNING, MF_BYCOMMAND | MF_UNCHECKED);
			else CheckMenuItem(sMenu, ID_DEMO_SELFRUNNING, MF_BYCOMMAND | MF_CHECKED);

	}
	return 1;
}

int CheckSDRAMSelect(HWND hWnd)
{
	HMENU hMenu, sMenu;

	hMenu = GetMenu(hWnd);
	if (hMenu==NULL)
	{
		MessageBox(hWnd, "main", "Registry sub-menu", MB_OK);
		return 0;
	}
	sMenu = GetSubMenu(hMenu, 0);	/* File */
	if (sMenu==NULL)
	{
		MessageBox(hWnd, "tools", "Registry sub-menu", MB_OK);
		return 0;
	}

	if (modsdram) CheckMenuItem(sMenu, IDD_SDRAMSELECT, MF_BYCOMMAND | MF_CHECKED);
	else          CheckMenuItem(sMenu, IDD_SDRAMSELECT, MF_BYCOMMAND);

	return 1;
}

int BitStreamBrowse(HWND hWnd)
{
	if (browseforbitstream(hWnd))
	{
		UpdateFileMenu(hWnd, mybitstream);
		return 1;
	}
	return 0;
}

int UpdateVarCoeffFuncMenu(HWND hWnd, int sdram)
{
	HMENU hMenu, sMenu;

	hMenu = GetMenu(hWnd);
	if (hMenu==NULL)
	{
		MessageBox(hWnd, "main", "Registry sub-menu", MB_OK);
		return 0;
	}
	sMenu = GetSubMenu(hMenu, 3);	/* Variable Coefficient Functions */
	if (sMenu==NULL)
	{
		MessageBox(hWnd, "tools", "Registry sub-menu", MB_OK);
		return 0;
	}

	if (sdram)
	{
		EnableMenuItem(sMenu, ID_DEMOMODE_LOAD, MF_ENABLED);
		EnableMenuItem(sMenu, ID_DEMOMODE_ADD , MF_ENABLED);
		EnableMenuItem(sMenu, ID_DEMOMODE_AND , MF_ENABLED);
		EnableMenuItem(sMenu, ID_DEMOMODE_OR  , MF_ENABLED);
		EnableMenuItem(sMenu, ID_DEMOMODE_SUB , MF_ENABLED);
		EnableMenuItem(sMenu, ID_DEMOMODE_XOR , MF_ENABLED);
		EnableMenuItem(sMenu, ID_DEMOMODE_MPY , MF_ENABLED);
	}
	else
	{
		EnableMenuItem(sMenu, ID_DEMOMODE_LOAD, MF_GRAYED);
		EnableMenuItem(sMenu, ID_DEMOMODE_ADD , MF_GRAYED);
		EnableMenuItem(sMenu, ID_DEMOMODE_AND , MF_GRAYED);
		EnableMenuItem(sMenu, ID_DEMOMODE_OR  , MF_GRAYED);
		EnableMenuItem(sMenu, ID_DEMOMODE_SUB , MF_GRAYED);
		EnableMenuItem(sMenu, ID_DEMOMODE_XOR , MF_GRAYED);
		EnableMenuItem(sMenu, ID_DEMOMODE_MPY , MF_GRAYED);
	}
	return 1;
}

/****************************************************/
/* SetBitMaps - create Windows bitmaps for images	*/
/****************************************************/
void SetBitMaps(int X, int Y)
{
	bmih.biWidth			= X;
	bmih.biHeight			= -(Y*2);

	// First bitmap - create 
//	RStatus = HeWaitForIo(hDevice, ReadIoStatus);
	DeleteObject(hBitmap1);
	hdcMem = CreateCompatibleDC(NULL);		// create DC
	hBitmap1 = CreateDIBSection( NULL, (BITMAPINFO *) &bmih, 0, (void **) &pimage1_bits, NULL, 0);
	SelectObject(hdcMem,hBitmap1);
	SetDIBColorTable(hdcMem, 0, 256, ColorTable);
	DeleteDC(hdcMem);

	// Second bitmap - create
	DeleteObject(hBitmap2);
	hdcMem = CreateCompatibleDC(NULL);		// create DC
	hBitmap2 = CreateDIBSection( NULL, (BITMAPINFO *) &bmih, 0, (void **) &pimage2_bits, NULL, 0);
	SelectObject(hdcMem,hBitmap2);
	SetDIBColorTable(hdcMem, 0, 256, ColorTable);
	DeleteDC(hdcMem);
}

LRESULT CALLBACK SlotNumber(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	int  wmId, wmEvent;

	switch (message)
	{
		case WM_INITDIALOG:
			switch (slotnumber)
			{
				case 1: CheckRadioButton(hDlg, IDC_SLOT1, IDC_SLOT4, IDC_SLOT1); break;
				case 2: CheckRadioButton(hDlg, IDC_SLOT1, IDC_SLOT4, IDC_SLOT2); break;
				case 3: CheckRadioButton(hDlg, IDC_SLOT1, IDC_SLOT4, IDC_SLOT3); break;
				case 4: CheckRadioButton(hDlg, IDC_SLOT1, IDC_SLOT4, IDC_SLOT4); break;
			}
			break;

		case WM_COMMAND:
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 
			switch (wmId)
			{
			case IDOK     :
				if (IsDlgButtonChecked(hDlg, IDC_SLOT1)==BST_CHECKED) slotnumber=1;
				else
				if (IsDlgButtonChecked(hDlg, IDC_SLOT2)==BST_CHECKED) slotnumber=2;
				else
				if (IsDlgButtonChecked(hDlg, IDC_SLOT3)==BST_CHECKED) slotnumber=3;
				else
				if (IsDlgButtonChecked(hDlg, IDC_SLOT4)==BST_CHECKED) slotnumber=4;
				EndDialog(hDlg, 1);
				return TRUE;

			case IDCANCEL :
				EndDialog(hDlg, 0);
				return TRUE;

			default : return FALSE;
			}
			break;
		default:
		    return FALSE;
	}
    return FALSE;
}



/****************************************************/
/* WinMain - main application code					*/
/****************************************************/

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
{
     HWND		hwnd;
     MSG		msg ;
     WNDCLASS	wndclass ;



    hInst = hInstance; // Store instance handle in our global variable

// Bitmap stuff
	unsigned	uNumEntries;
	long RW;

	 // Set up our window class
	 // We'll register this as the windows we create
     wndclass.style         = 0 | CS_HREDRAW | CS_VREDRAW | CS_BYTEALIGNCLIENT ;
     wndclass.lpfnWndProc   = WndProc ;
     wndclass.cbClsExtra    = 0 ;
     wndclass.cbWndExtra    = 0 ;
     wndclass.hInstance     = hInstance ;
     wndclass.hIcon         = LoadIcon (NULL, IDI_APPLICATION) ;
     wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
     wndclass.hbrBackground = NULL ;	// setting to NULL means there's no background
										// other values = blinking between frames
//     wndclass.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH) ;
     wndclass.lpszMenuName  = szAppName ;
     wndclass.lpszClassName = szAppName ;




	 // Check to see if we can register a window
	 // This test fails in the debugger - don't know why
     if (!RegisterClass (&wndclass))
     {
          MessageBox (NULL, TEXT ("This program requires 32-bit Windows!"), 
                      szAppName, MB_ICONERROR) ;
          return 0 ;
     }

	 // Create the window
	 // Size and position set to default
    hwnd = CreateWindow (szAppName, TEXT ("Imaging Demo Display"),
						  WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU | 
						  WS_MINIMIZEBOX | WS_BORDER,
                          50, 50,
                          (800+(IMAGE_X-384)), (600+IMAGE_Y-384), 
                          NULL, NULL, hInstance, NULL) ;



	// Draw an empty window...
	ShowWindow (hwnd, iCmdShow) ;
	UpdateWindow (hwnd) ;
	UpdateVarCoeffFuncMenu(hwnd, modsdram);


// Create two bitmap sections for the images from the board
// We alternate between the images so while one is displayed the other 
// is receiving an image from the board
    bmih.biSize			= sizeof(BITMAPINFOHEADER);
	bmih.biWidth			= IMAGE_X;
	bmih.biHeight			= -(IMAGE_Y*2);
	bmih.biPlanes			= 1;
	bmih.biBitCount			= 8; 
	bmih.biCompression		= BI_RGB;
 	bmih.biSizeImage		= 0; 
 	bmih.biXPelsPerMeter	= 0; 
 	bmih.biYPelsPerMeter	= 0; 
 	bmih.biClrUsed			= 0; 
 	bmih.biClrImportant		= 0; 


	// Initialise colour tables
	for(uNumEntries=0;uNumEntries<256;uNumEntries++)
	{
		ColorTable[uNumEntries].rgbRed=uNumEntries;
		ColorTable[uNumEntries].rgbGreen=uNumEntries;
		ColorTable[uNumEntries].rgbBlue=uNumEntries;
	}

	SetBitMaps(IMAGE_X, IMAGE_Y);



	// create images with test pattern
	for (RW=0;RW<IMAGE_X*IMAGE_Y;RW++)
		pimage1_bits[RW] = 0xff;
	for (RW=0;RW<IMAGE_X*IMAGE_Y;RW++)
		pimage2_bits[RW] = RW&0xff;

	// set first bitmap for display, mark screen for update
	 hBitmap_Disp=hBitmap1;
     InvalidateRect (hwnd, NULL, TRUE) ;

	startsem = CreateSemaphore(NULL, 0, 1, "start");
	if (startsem==NULL)
	{
		MessageBox(NULL, "Unable to create sempahore. Aborting.", "Imaging Demo", MB_OK);
		return 0;
	}


	/* Start second thread for board I/O */
	params.hwnd			= hwnd;
	params.bContinue	= TRUE;
	hThread = CreateThread( NULL, 0x200000, (LPTHREAD_START_ROUTINE) &Thread , &params, 0, &DSP_ThreadID );
	SetThreadPriority(hThread,THREAD_PRIORITY_ABOVE_NORMAL);


   SetTimer(hwnd, 2, 10000, Statsfunc);
// Read messages and process them
// continuous loop - read message, post, process...


	while(GetMessage (&msg, NULL, 0, 0))
	{
		TranslateMessage (&msg) ;
		DispatchMessage (&msg) ;
	}

     return msg.wParam ;
}



/****************************************************************/
/* WndPROC window message handler								*/
/****************************************************************/
LRESULT CALLBACK WndProc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
     static BITMAP		bitmap ;
     HDC				hdc, hdcMem ;
     PAINTSTRUCT		ps ;
	 static RECT		rect;
	 static RECT		textbox;
	 HMENU				hMenu;
	 int	status;
	 UINT	DSP_message[20];


	 DSP_message[0]=0;
     switch (message)
     {
	 // sent when window created
	 // Screen_Draw flag requests screen blank
     case WM_CREATE:
		 Screen_Draw=TRUE;
        return 0 ;

	 // sent when new image available for display
	 // invalidates display, forcing a WM_PAINT event
     case FPGA_IMAGE:
		  InvalidateRect (hwnd, NULL, TRUE) ;
          return 0 ;

	 // sent when new text available for status
	 // sets Status_changed flag
	 // invalidates display, forcing a WM_PAINT event
     case FPGA_STATUS:
		  Status_Changed=TRUE;
		  InvalidateRect (hwnd, NULL, TRUE) ;
          return 0 ;
     
	 case WM_COMMAND:
		 {
			 hMenu=GetMenu(hwnd);
			 switch (LOWORD(wParam))
				 {
					 case IDD_BITSTREAM:
						BitStreamBrowse(hwnd);
						break;
					 case IDD_SLOT:
						DialogBox(hInst, (LPCTSTR)IDD_SLOTSELECT, hwnd, (DLGPROC)SlotNumber);
						break;
					 case IDD_START:
						comment_pointer = NO_COMMENT;
						ReleaseSemaphore(startsem, 1, NULL);
						break;
					 case IDD_STOP:
						status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], HALTTHREAD, 0L);
						break;
					 case IDD_SDRAMSELECT:
						modsdram=!modsdram;
						CheckSDRAMSelect(hwnd);
						UpdateVarCoeffFuncMenu(hwnd, modsdram);
						break;


					// sending message to DSP
					 case ID_DEMO_SELFRUNNING:
							if(demo_runmode==0)
							{
								demo_runmode=1;
								SetTimer(hwnd, 1, 3000, NULL);
								CheckMenuItem( hMenu,LOWORD(wParam), MF_CHECKED);
							}
							else
							{
								demo_runmode=0;
								KillTimer(hwnd,1);
								CheckMenuItem( hMenu,LOWORD(wParam), MF_UNCHECKED);
							}
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_NONE:
							demo_mode=NONE;
							comment_pointer = NO_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], NONE, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_NOT:
							demo_mode=NOT;
							comment_pointer = NOT_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], NOT, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_ANDK:
							demo_mode=ANDK;
							comment_pointer = ANDK_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], ANDK, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_SQU:
							demo_mode=SQU;
							comment_pointer = SQU_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], SQU, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_MPYK:
							demo_mode=MPYK;
							comment_pointer = MPYK_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], MPYK, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_RSHK:
							demo_mode=RSHK;
							comment_pointer = RSHK_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], RSHK, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_LSHK:
							demo_mode=LSHK;
							comment_pointer = LSHK_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], LSHK, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_ADDK:
							demo_mode=ADDK;
							comment_pointer = ADDK_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], ADDK, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_ORK:
							demo_mode=ORK;
							comment_pointer = ORK_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], ORK, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_XORK:
							demo_mode=XORK;
							comment_pointer = XORK_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], XORK, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_SUBK:
							demo_mode=SUBK;
							comment_pointer = SUBK_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], SUBK, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_FILLK:
							demo_mode=FILLK;
							comment_pointer = FILLK_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], FILLK, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_RAMP:
							demo_mode=RAMP;
							comment_pointer = RAMP_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], RAMP, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_LOAD:
							demo_mode=LOAD;
							comment_pointer = LOAD_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], LOAD, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_ADD:
							demo_mode=ADD;
							comment_pointer = ADD_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], ADD, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_AND:
							demo_mode=AND;
							comment_pointer = AND_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], AND, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_OR:
							demo_mode=OR;
							comment_pointer = OR_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], OR, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_SUB:
							demo_mode=SUB;
							comment_pointer = SUB_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], SUB, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_XOR:
							demo_mode=XOR;
							comment_pointer = XOR_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], XOR, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_MPY:
							demo_mode=MPY;
							comment_pointer = MPY_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], MPY, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_vSOBEL:
							demo_mode=vSOBEL;
							comment_pointer = VSOBEL_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], vSOBEL, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_hSOBEL:
							demo_mode=hSOBEL;
							comment_pointer = HSOBEL_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], hSOBEL, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_vPREWITT:
							demo_mode=vPREWITT;
							comment_pointer = VPREWITT_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], vPREWITT, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_hPREWITT:
							demo_mode=hPREWITT;
							comment_pointer = HPREWITT_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], hPREWITT, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_LAPLACIAN:
							demo_mode=LAPLACIAN;
							comment_pointer = LAPLACE_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], LAPLACIAN, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_LPFILTER:
							demo_mode=LPFILTER;
							comment_pointer = LOWPASS_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], LPFILTER, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_SHARPEN:
							demo_mode=SHARPEN;
							comment_pointer = SHARPEN_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], SHARPEN, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_vSOBEL5X5:
							demo_mode=vSOBEL5X5;
							comment_pointer = VSOBEL_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], vSOBEL5X5, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_hSOBEL5X5:
							demo_mode=hSOBEL5X5;
							comment_pointer = HSOBEL_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], hSOBEL5X5, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 case ID_DEMOMODE_LAPLACIAN5X5:
							demo_mode=LAPLACIAN5X5;
							comment_pointer = LAPLACE_COMMENT;
							DSP_message[0]=DSPCMD_MODE;
							status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], LAPLACIAN5X5, 0L);
							Screen_Draw=TRUE;
							InvalidateRect (hwnd, NULL, TRUE) ;
							return 0 ;
					 default:
							return 0 ;
				 }
		 }

     case WM_TIMER:
			if (demo_runmode)
			{
			//	MessageBeep(-1);
				DSP_message[0]=DSPCMD_MODE;
				if(demo_mode==NONE)
				{
					demo_mode = NOT;
					comment_pointer = NOT_COMMENT;
				}
				else if(demo_mode==NOT)
				{
					demo_mode = ANDK;
					comment_pointer = ADDK_COMMENT;
				}
				else if(demo_mode==ANDK)
				{
					demo_mode = SQU;
					comment_pointer = SQU_COMMENT;
				}
				else if(demo_mode==SQU)
				{
					demo_mode = MPYK;
					comment_pointer = MPYK_COMMENT;
				}
				else if(demo_mode==MPYK)
				{
					demo_mode = RSHK;
					comment_pointer = RSHK_COMMENT;
				}
				else if(demo_mode==RSHK)
				{
					demo_mode = LSHK;
					comment_pointer = LSHK_COMMENT;
				}
				else if(demo_mode==LSHK)
				{
					demo_mode = ADDK;
					comment_pointer = ADDK_COMMENT;
				}
				else if(demo_mode==ADDK)
				{
					demo_mode = ORK;
					comment_pointer = ORK_COMMENT;
				}
				else if(demo_mode==ORK)
				{
					demo_mode = XORK;
					comment_pointer = XORK_COMMENT;
				}
				else if(demo_mode==XORK)
				{
					demo_mode = SUBK;
					comment_pointer = SUBK_COMMENT;
				}
				else if(demo_mode==SUBK)
				{
					demo_mode = FILLK;
					comment_pointer = FILLK_COMMENT;
				}
				else if(demo_mode==FILLK)
				{
					demo_mode = RAMP;
					comment_pointer = RAMP_COMMENT;
				}
				else if(demo_mode==RAMP)
				{
					if (modsdram)
					{
					demo_mode = LOAD;
					comment_pointer = LOAD_COMMENT;
					}
					else
					{
					demo_mode = vSOBEL;
					comment_pointer = VSOBEL_COMMENT;
					}
				}
				else if(demo_mode==LOAD)
				{
					demo_mode = ADD;
					comment_pointer = ADD_COMMENT;
				}
				else if(demo_mode==ADD)
				{
					demo_mode = AND;
					comment_pointer = AND_COMMENT;
				}
				else if(demo_mode==AND)
				{
					demo_mode = OR;
					comment_pointer = OR_COMMENT;
				}
				else if(demo_mode==OR)
				{
					demo_mode = SUB;
					comment_pointer = SUB_COMMENT;
				}
				else if(demo_mode==SUB)
				{
					demo_mode = XOR;
					comment_pointer = XOR_COMMENT;
				}
				else if(demo_mode==XOR)
				{
					demo_mode = MPY;
					comment_pointer = MPY_COMMENT;
				}
				else if(demo_mode==MPY)
				{
					demo_mode = vSOBEL;
					comment_pointer = VSOBEL_COMMENT;
				}
				else if(demo_mode==vSOBEL)
				{
					demo_mode = hSOBEL;
					comment_pointer = HSOBEL_COMMENT;
				}
				else if(demo_mode==hSOBEL)
				{
					demo_mode = vPREWITT;
					comment_pointer = VPREWITT_COMMENT;
				}
				else if(demo_mode==vPREWITT)
				{
					demo_mode = hPREWITT;
					comment_pointer = HPREWITT_COMMENT;
				}
				else if(demo_mode==hPREWITT)
				{
					demo_mode = LAPLACIAN;
					comment_pointer = LAPLACE_COMMENT;
				}
				else if(demo_mode==LAPLACIAN)
				{
					demo_mode = LPFILTER;
					comment_pointer = LOWPASS_COMMENT;
				}
				else if(demo_mode==LPFILTER)
				{
					demo_mode = SHARPEN;
					comment_pointer = SHARPEN_COMMENT;
				}
				else
				{
					demo_mode = NONE;
				    comment_pointer = NO_COMMENT;
				}
				status = PostThreadMessage( (DWORD) DSP_ThreadID, DSP_message[0], demo_mode, 0L);
				Screen_Draw=TRUE;
				InvalidateRect (hwnd, NULL, TRUE) ;
			}
			return 0;


     case WM_PAINT:
          hdc = BeginPaint (hwnd, &ps) ;
          hdcMem = CreateCompatibleDC (hdc) ;
		  // if status has changed...
		  if(Status_Changed)
		  {
				GetClientRect(hwnd,&rect);
				Rectangle(hdc,0,(535+IMAGE_X-384),rect.right,(560+IMAGE_Y-384));
				TextOut(hdc,3,(538+IMAGE_X-384),Status_Text, strlen(Status_Text));
				Status_Changed = FALSE;
		  }

		  if(Screen_Draw==TRUE)
		  {

			GetClientRect(hwnd,&rect);
			Rectangle(hdc,0,0,rect.right,rect.bottom);
			Screen_Draw=FALSE;
/***************************************************************/
			GetClientRect(hwnd,&rect);
			Rectangle(hdc,0,(535+IMAGE_X-384),rect.right,(560+IMAGE_Y-384));
			TextOut(hdc,3,(538+IMAGE_X-384),Status_Text, strlen(Status_Text));
			Status_Changed = FALSE;
/***************************************************************/
			textbox.left =	500+IMAGE_X-384;
			textbox.right = 750+IMAGE_Y-384;
			textbox.top = 72;
			textbox.bottom = 76+IMAGE_Y;
            Rectangle(hdc,74,74,76+IMAGE_X, 76+IMAGE_Y);


			DrawTextEx(  hdc,comment_pointer,-1,
						&textbox,DT_WORDBREAK,NULL);

			SetTextAlign(hdc,TA_CENTER);
			TextOut(hdc,400,1,"HUNT ENGINEERING", strlen("HUNT ENGINEERING") );
			TextOut(hdc,400,20,"FPGA Imaging Demo", strlen("FPGA Imaging Demo"));
		  }
          SelectObject (hdcMem, hBitmap_Disp) ;
          BitBlt (hdc,    75, 75, IMAGE_X, IMAGE_Y,hdcMem, 0, 0, SRCCOPY) ;
          DeleteDC (hdcMem) ;
          EndPaint (hwnd, &ps) ;
          return 0 ;
          
	 case	WM_SETFOCUS:
	 case	WM_SHOWWINDOW:
	 case	WM_ACTIVATE:
	 case	WM_ENABLE:
	 case	WM_CHILDACTIVATE:
	 case	WM_ACTIVATEAPP:
	 case	WM_LBUTTONDOWN:
			Screen_Draw=TRUE;
			InvalidateRect (hwnd, NULL, TRUE) ;
			return 0 ;

     case WM_DESTROY:
		  KillTimer(hwnd, 1);
		  KillTimer(hwnd, 2);
          PostQuitMessage (0) ;
          return 0 ;

	 }
     return DefWindowProc (hwnd, message, wParam, lParam) ;
}




int ParseKeyword(char *line, char *key, int *val)
{
	int i, j, l, k;
	int ok=0, v, n, base;

	l = strlen(line);
	k = strlen(key);
	for (i=0; i<(l-k+1); i++)
	{
		/* If comment then quit */
		if ( line[i]=='#'                   ) return 0;
		if ((line[i]=='/')&&(line[i+1]=='/')) return 0;
		if ( line[i]=='!'                   ) return 0;

		if (tolower(line[i])==tolower(key[0]))
		{
			ok=1;
			for (j=1; j<k; j++)
			{
				if (tolower(line[i+j])!=tolower(key[j]))
				{
					ok=0;
				}
			}
		}
		if (ok) break;
	}

	if (!ok) return 0;

	/* Move on to the '=' sign */
	while ((line[i]!=0)&&(i<l)&&(line[i]!='=')) i++;
	if (line[i]!='=') return 0;
	i++;

	/* Read the number */
	base = 10;
	n    = 0;
	v    = 0;
	while (ok)
	{
		switch (line[i])
		{
		case '0': if ((line[i+1]=='x')||(line[i+1]=='X'))
				  {
					  base = 16; i++;
				  }
				  else
				  {
					  n = n*base+0; v=1;
				  }
				  break;
		case '1': n = n*base+1; v=1; break;
		case '2': n = n*base+2; v=1; break;
		case '3': n = n*base+3; v=1; break;
		case '4': n = n*base+4; v=1; break;
		case '5': n = n*base+5; v=1; break;
		case '6': n = n*base+6; v=1; break;
		case '7': n = n*base+7; v=1; break;
		case '8': n = n*base+8; v=1; break;
		case '9': n = n*base+9; v=1; break;
		case 'a':
		case 'A': if (base<10) { *val = n; return v; }
			      n = n*base+10; v=1;
				  break;
		case 'b':
		case 'B': if (base<10) { *val = n; return v; }
			      n = n*base+11; v=1;
				  break;
		case 'c':
		case 'C': if (base<10) { *val = n; return v; }
			      n = n*base+12; v=1;
				  break;
		case 'd':
		case 'D': if (base<10) { *val = n; return v; }
			      n = n*base+13; v=1;
				  break;
		case 'e':
		case 'E': if (base<10) { *val = n; return v; }
			      n = n*base+14; v=1;
				  break;
		case 'f':
		case 'F': if (base<10) { *val = n; return v; }
			      n = n*base+15; v=1;
				  break;
		case ' ':
		case  9 :
		case 10 :
		case 11 :
		case 13 : break;
		default : *val=n; return v;
		}
		i++;
	}

	return 0;
}

void ReadCameraFile(CAMERA *d)
{
	FILE *f;
	char *s;
	char  line[1024];
	int   more, val;

	d->cam_control  = CAM_CONTROL;
	d->cam_use_dval = CAM_USE_DVAL;
	d->cam_polarity = CAM_POLARITY;
	d->visible      = 1;


	sprintf(line,"%s\\camera.dat",exepath);
	f = fopen(line, "r");
	if (f==NULL)
	{
		strcat("Cannot find or open file",line);
		MessageBox(NULL, line, "Imaging Demo", MB_OK);
		exit(0);
	}
	else
	{
		more=1;
		while (more)
		{
			s = fgets(line, 1024, f);
			if (ferror(f))
			{
				MessageBox(NULL, "Error reading \"camera.dat\" file. Abort.", "Imaging Demo", MB_OK);
				exit(0);
			}
			if (feof(f))
			{
				more=0;
			}
			if (ParseKeyword(line, "cam_control", &val))
			{
				d->cam_control = val;
			}
			if (ParseKeyword(line, "cam_use_dval", &val))
			{
				d->cam_use_dval = val;
			}
			if (ParseKeyword(line, "cam_polarity", &val))
			{
				d->cam_polarity = val;
			}
			if (ParseKeyword(line, "visible", &val))
			{
				d->visible = val;
			}
		}
		fclose(f);
	}
}

/****************************************************************/
/* FPGA management thread										*/
/****************************************************************/

/* This is our function that will replace user_vprint of the S/L */
void hc_vprint(char *msg)
{
	/* hrn_fpga sends "\r%% Done: %d" verbose messages, but  */
	/* other strings as well. We want to replace "\r" by our */
	/* own string but let other strings pass unchanged.      */
	if (msg[0]=='\r') sprintf(Status_Text,"Loading FPGA: %s", &msg[1]);
	else sprintf(Status_Text,"%s", msg);
	PostMessage(pparams->hwnd,FPGA_STATUS,0,0);
}

int run_demo(PVOID pvoid);

long WINAPI Thread(PVOID pvoid)
{
	while(1)
	{
		WaitForSingleObject(startsem, INFINITE);

		while (mybitstream[0]==0)
		{
			if (mybitstream[0]==0)
			{
				MessageBox(NULL, "Please select a bitstream first.", "Imaging Demo", MB_OK);
			}
			WaitForSingleObject(startsem, INFINITE);
		}

		run_demo(pvoid);		
	}
}

int run_demo(PVOID pvoid)
{
	int r=0, more = 1;
	int localstatus;
	int n,m,i,j,length,start,ask;
    MSG      msg ;
	unsigned long	FPGA_command[20];
	FILE *fptr;
	char  command[1024];
	char  netpath[1024];
	char *boardString;
	char *commandString;
	char  tmp[1024];

    commandString = NULL;
	boardString = NULL;

	HE_DWORD *frame1, *frame2, *currentFrame;
	HE_BYTE  *currentbyteFrame;

	int c;
	HE_DWORD read_length;
	HE_DWORD adjust,skipframe;
	HE_DWORD index;



	read_length = (IMAGE_Y * ((IMAGE_X / 4) + 1));
	adjust = 0;
	runstats=0;

	// get parameter structure
	// set status bar
	PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE);
	pparams=(PPARAMS) pvoid;

    /* open the board and reset it */
	localstatus = HeOpen1(devname, (HE_WORD)Board, "FifoA", &hDevice);
	if(localstatus != HE_OK) return error("Open Failed.", localstatus);
	strcpy(Status_Text,"Board Open....");
	PostMessage(pparams->hwnd,FPGA_STATUS,0,0);
    
	/* reset the hardware*/
	HeReset(hDevice);
	strcpy(Status_Text,"Board Reset....");
	PostMessage(pparams->hwnd,FPGA_STATUS,0,0);
	HeClose(&hDevice);

	/* run hrn_fpga */
	strcpy(Status_Text,"Setting up FPGA....");
	PostMessage(pparams->hwnd,FPGA_STATUS,0,0);

#if 0
	/* With Windows 98 the code below doesn't work if the commandline */
	/* is longer than 122 characters. Which happens with the code on  */
	/* the CD. We thus replaced it with the DLL version of hrn_fpga.  */
	commandString=getenv("HEAPI_DIR");

	sprintf(command,"%s\\utils\\hrn_fpga -u -f%s -s%d\n",commandString,mybitstream,slotnumber);
	system(command);
#else
	{
		hrn_fpga hf;
		char     mesg[1024];

		hf.hrn_fpga_set_userprint(hc_vprint);
	
		r = hf.hrn_fpga_board(devname, Board, slotnumber, mybitstream, HRN_FPGA_LOAD, HRN_FPGA_VERBOSE);
		if (r)
		{
			char *s;

			s = hf.hrn_fpga_getlasterr();
			if ((s==NULL)||(s[0]==0))
			{
				sprintf(mesg, "Hrn_fpga reports error %d.", r);
			}
			else
			{
				sprintf(mesg, "Hrn_fpga: %s", s);
			}
			strcpy(Status_Text,mesg);
			PostMessage(pparams->hwnd,FPGA_STATUS,0,0);
			return error(mesg, HE_OK);
		}
	}
#endif

	strcpy(Status_Text,"FPGA loaded");
	PostMessage(pparams->hwnd,FPGA_STATUS,0,0);

	ask = 0;
	if (ask==0)
	{
		/* Set up for board we're going to open */
		boardString=getenv("HEAPIHSB");
		if (boardString==NULL)
		{	
			if (verbose)printf("No HEAPIHSB variable found, using HEAPIJTAG\n");
			boardString=getenv("HEAPIJTAG");
			if (boardString==NULL)
			{
				if (verbose)printf("No HEAPIJTAG variable found\n");
			}
		}
	}	

	if (boardString != NULL)
	{
		i = 0;
		while ((boardString[i]==' ') || (boardString[i]=='\t'))i++;
		start=i;
		while ((boardString[i]!=' ') && (boardString[i]!='\t'))i++;
		length=i-start; 
		strncpy(devname,&boardString[start] , length);
		i++;
		while ((boardString[i]==' ') || (boardString[i]=='\t'))i++;
		start=i;
		while ((boardString[i]!=' ') && (boardString[i]!='\t'))i++;
		length=i=start;
		strncpy(tmp,&boardString[start] , length);
		Board   = (HE_WORD)atoi(tmp);
	}


	/*************** Initialise HSB ******************/

    /* open the board */
	strcpy(Status_Text,"Board Open....");
	PostMessage(pparams->hwnd,FPGA_STATUS,0,0);
	localstatus = HeOpen1(devname, (HE_WORD)Board, "FifoA", &hDevice);
	if(localstatus != HE_OK) return error("Open Failed.", localstatus);
    
	/* reset the hardware*/
	strcpy(Status_Text,"Board Reset....");
	PostMessage(pparams->hwnd,FPGA_STATUS,0,0);
	HeReset(hDevice);

	/*find the directory that we are in*/
	strcpy(exepath,GetCommandLine());
	
	/* remove the .exe name*/
	i=strlen(exepath);
	while (exepath[i]!='\\')
	{
		i--;
	}
	exepath[i]=0;
	/*remove the initial quotes*/
	for(j=0;j<i;j++)
	{
		exepath[j]=exepath[j+1];
	}
	if (stricmp(devname,"hep8a")!=0)
	{
		strcpy(Status_Text,"Setting the HEART configuration on HEPC9");
		PostMessage(pparams->hwnd,FPGA_STATUS,0,0);

		/* find API installation directory */
		commandString=getenv("HEAPI_DIR");

		/* check to see if a network file exists in this directory*/
		sprintf(command,"%s\\network",exepath);

		fptr=fopen(command, "r");	

		if (fptr!=NULL)
		{
			fclose(fptr);
			sprintf(netpath,"%s\\network",exepath);
			sprintf(command,"%s\\heartconf\\heartconf %s\n",commandString,netpath);
		}
		else 
		{
			/* if not then look in the directory above in case we are running from a debug or release directory*/ 
			sprintf(command,"%s\\..\\network",exepath);
			fptr=fopen(command, "r");	
			if (fptr!=NULL)
			{
				fclose(fptr);
				sprintf(netpath,"%s\\..\\network",exepath);
				sprintf(command,"%s\\heartconf\\heartconf %s\n",commandString,netpath);
			}
			else
			{
				sprintf(Status_Text,"This directory: '%s' (file 'network').", exepath);
				PostMessage(pparams->hwnd,FPGA_STATUS,0,0);
				return error("Cannot locate the network file in this or the directory above.", HE_OK);
			}
		}
#if 0
		/* call heart conf to set the network up */		
		system(command);
#else
		{
			hesl sl;

			/* Run server-loader ---------------- */
			r = sl.heartconf("-v", netpath);
			if (r)
			{
				char *s;
				char  msg[256];

				s = sl.getlasterr();
				if ((s==NULL)||(s[0]==0))
				{
					sprintf(msg, "HeartConf reports error %d.", r);
				}
				else
				{
					sprintf(msg, "HeartConf. %s", s);
				}
				strcpy(Status_Text,msg);
				PostMessage(pparams->hwnd,FPGA_STATUS,0,0);
				return error(msg, HE_OK);
			}
		}
#endif
	}
	else /* is an HEPC8*/
	{
		slotnumber=1;
	}

	switch (slotnumber){
		case 1: break;
		case 2: localstatus = HeClose(&hDevice);
				localstatus = HeOpen1(devname, (HE_WORD)Board, "FifoB", &hDevice);
				if(localstatus != HE_OK) return error("Open Failed.", localstatus);
				break;
		case 3: localstatus = HeClose(&hDevice);
				localstatus = HeOpen1(devname, (HE_WORD)Board, "FifoC", &hDevice);
				if(localstatus != HE_OK) return error("Open Failed.", localstatus);
				break;
		case 4: localstatus = HeClose(&hDevice);
				localstatus = HeOpen1(devname, (HE_WORD)Board, "FifoD", &hDevice);
				if(localstatus != HE_OK) return error("Open Failed.", localstatus);
				break;
		default: break;
	}


	ReadIoStatus=NULL;
	localstatus = HeInitIoStatus(hDevice, &ReadIoStatus); // Initialise read IoStatus
	if(localstatus != HE_OK) return error("Can't init Read I/O.", localstatus);

	/* send messages over the HSB to set the registers in the FPGA that select the HERON FIFO */
	/* that the FPGA will use to read and write */

	ReadCameraFile(&d);





    /* Open Board */
	localstatus = Initialise_Host_HSB(devname,Board);
	if(localstatus != HE_OK) return error("Can't init HSB.", localstatus);
	strcpy(Status_Text,"Opening Board....");
	PostMessage(pparams->hwnd,FPGA_STATUS,0,0);



SendUserData(devname, Board, slotnumber, FIFO_NO_ADDR, 0x01);

strcpy(Status_Text,"FPGA FIFO's set....");
PostMessage(pparams->hwnd,FPGA_STATUS,0,0);


hsb_setup(devname, Board, slotnumber,&d);

/**********************************************************/
//Start testing
/**********************************************************/
strcpy(Status_Text,"Running....");

PostMessage(pparams->hwnd,FPGA_STATUS,0,0);

if((frame1 = (HE_DWORD *)malloc((IMAGE_Y * ((IMAGE_X/4) + 1)) * sizeof(HE_DWORD))) == NULL) {
	printf("Error: call to Malloc failed\n");
	exit(0);
}
if((frame2 = (HE_DWORD *)malloc((IMAGE_Y * ((IMAGE_X/4) + 1)) * sizeof(HE_DWORD))) == NULL) {
	printf("Error: call to Malloc failed\n");
	exit(0);
}
/*************************************************/

//SendUserData(devname, Board, slotnumber, MODE_REG_ADDR, 0x14);

/* Frame control */
SendUserData(devname, Board, slotnumber, FRAME_CONTROL_ADDR, 0x10);
j = 0;

	runstats=1;
	int nomore     = 0;

	/*prevent the waitio the first time*/
	localstatus=HE_OK;
	skipframe=0;

	while (!nomore)
	{
	
		/************** Start Reading results **********************/
		index = (read_length - adjust);
	
	/* wait for the last frame to completely arrive*/
	if (localstatus!=HE_OK) HeWaitForIo(hDevice, ReadIoStatus);	

/* re-start the reading as soon as possible*/

		if(hBitmap_Disp==hBitmap1)
		{
			hBitmap_Disp=hBitmap2;
			/* indicate to use the other one for display*/
			currentFrame = frame2;
	        currentBitMap = pimage2_bits;
			/*start the read into this one*/
			localstatus = HeRead(hDevice, frame1, index * sizeof(HE_DWORD), ReadIoStatus);
			if((localstatus != HE_OK) && (localstatus != HE_IoInProgress))
				return error("Failed to read1", localstatus);
		}
		else
		{
			hBitmap_Disp=hBitmap1;
			/* indicate to use the other one for display*/
			currentFrame = frame1;
			currentBitMap = pimage1_bits;
			/*start the read into this one*/
			localstatus = HeRead(hDevice, frame2, index * sizeof(HE_DWORD), ReadIoStatus);
			if((localstatus != HE_OK) && (localstatus != HE_IoInProgress))
				return error("Failed to read2", localstatus);
		}


		/* now the read is in progress, we have some time to check the frame and display it*/
		/* if we take too long here then we'll fail to read the FIFO and data will be lost*/

		index--; /* because array addressing goes from 0*/




	    // One Frame received. Check last word for sync info

	  	adjust = 0;
		c=0;

		if(skipframe==0)
		{
			if (currentFrame[index] == 0xFFFFFFFF)
			{
				// Frame sync is perfect, so display frame
				currentbyteFrame=(HE_BYTE * )currentFrame;
				for(m=0;m<(IMAGE_Y);m++) 
				{
					for(n=0;n<(IMAGE_X);n++) 
					{
						currentBitMap[c++]=(int)currentbyteFrame[(m*(IMAGE_X+4))+n]; /* +4 because of extra sync word*/
					}
				}
				/* make the display window update */ 
				PostMessage(pparams->hwnd,FPGA_IMAGE,0,0);
				goodframes++;
			}
  			else
			{
  				// Look for frame sync value elsewhere 
				while((currentFrame[index-adjust]!=0xFFFFFFFF)&&(adjust<(index))) adjust++;
				if (adjust==index)	adjust=0; /* sync not found*/
				else skipframe=2; /* don't check the next 2 frames while the sync takes effect*/
			}
		}
		else
		{
		skipframe--;
		}

		/*********** Check for messages *****************/
		if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
		{
			FPGA_command[0] = msg.message;
			FPGA_command[1] = msg.wParam;
			FPGA_command[2] = msg.lParam;

			if(FPGA_command[1] == NONE)
			{
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x00);
			}
			else
			if(FPGA_command[1] == NOT)
			{
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x01);
			}
			else
			if(FPGA_command[1] == ANDK)
			{
				SendUserData(devname, Board, slotnumber, CAT1_CONST, 0xAA);
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x02);
			}
			else
			if(FPGA_command[1] == SQU)
			{
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x03);
			}
			else
			if(FPGA_command[1] == MPYK)
			{
				SendUserData(devname, Board, slotnumber, CAT1_CONST, 0x04);
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x04);
			}
			else
			if(FPGA_command[1] == RSHK)
			{
				SendUserData(devname, Board, slotnumber, CAT1_CONST, 0x01);
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x05);
			}
			else
			if(FPGA_command[1] == LSHK)
			{
				SendUserData(devname, Board, slotnumber, CAT1_CONST, 0x03);
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x06);
			}
			else
			if(FPGA_command[1] == ADDK)
			{
				SendUserData(devname, Board, slotnumber, CAT1_CONST, 0x80);
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x07);
			}
			else
			if(FPGA_command[1] == ORK)
			{
				SendUserData(devname, Board, slotnumber, CAT1_CONST, 0xAA);
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x08);
			}
			else
			if(FPGA_command[1] == XORK)
			{
				SendUserData(devname, Board, slotnumber, CAT1_CONST, 0xAA);
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x09);
			}
			else
			if(FPGA_command[1] == SUBK)
			{
				SendUserData(devname, Board, slotnumber, CAT1_CONST, 0x80);
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x0A);
			}
			else
			if(FPGA_command[1] == FILLK)
			{
				SendUserData(devname, Board, slotnumber, CAT1_CONST, 0xAA);
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x0B);
			}
			else
			if(FPGA_command[1] == RAMP)
			{
				SendUserData(devname, Board, slotnumber, CAT1_CONST, 0x1F);
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x0C);
			}
			else
			if(FPGA_command[1] == LOAD)
			{
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x0D);
			}
			else
			if(FPGA_command[1] == ADD)
			{
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x0E);
			}
			else
			if(FPGA_command[1] == AND)
			{
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x0F);
			}
			else
			if(FPGA_command[1] == OR)
			{
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x10);
			}
			else
			if(FPGA_command[1] == SUB)
			{
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x11);
			}
			else
			if(FPGA_command[1] == XOR)
			{
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x12);
			}
			else
			if(FPGA_command[1] == MPY)
			{
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x13);
			}
			else
			if(FPGA_command[1] == vSOBEL)
			{
				setConvolutionConstants(devname, Board, slotnumber, FPGA_command[1]);
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x14);
			}
			else
			if(FPGA_command[1] == hSOBEL)
			{
				setConvolutionConstants(devname, Board, slotnumber, FPGA_command[1]);
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x14);
			}
			else
			if(FPGA_command[1] == vPREWITT)
			{
				setConvolutionConstants(devname, Board, slotnumber, FPGA_command[1]);
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x14);
			}
			else
			if(FPGA_command[1] == hPREWITT)
			{
				setConvolutionConstants(devname, Board, slotnumber, FPGA_command[1]);
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x14);
			}
			else
			if(FPGA_command[1] == LAPLACIAN)
			{
				setConvolutionConstants(devname, Board, slotnumber, FPGA_command[1]);
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x14);
			}
			else
			if(FPGA_command[1] == LPFILTER)
			{
				setConvolutionConstants(devname, Board, slotnumber, FPGA_command[1]);
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x14);
			}
			else
			if(FPGA_command[1] == SHARPEN)
			{
				setConvolutionConstants(devname, Board, slotnumber, FPGA_command[1]);
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x14);
			}
			else
			if(FPGA_command[1] == vSOBEL5X5)
			{
				setConvolutionConstants(devname, Board, slotnumber, FPGA_command[1]);
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x14);
			}
			else
			if(FPGA_command[1] == hSOBEL5X5)
			{
				setConvolutionConstants(devname, Board, slotnumber, FPGA_command[1]);
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x14);
			}
			else
			if(FPGA_command[1] == LAPLACIAN5X5)
			{
				setConvolutionConstants(devname, Board, slotnumber, FPGA_command[1]);
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x14);
			}
			else
			if(FPGA_command[1] == HALTTHREAD)
			{
				nomore=1;
			}
			else
			{
				SendUserData(devname, Board, slotnumber, IMAGE_BLOCK_SEL, 0x00);
			}
		}


	}

	/* Close down API and return */
	Status = HeClose(&hDevice);
	if(Status != HE_OK) return error("Failed close.", Status);
  	Status = HeClose(&hsbDevice);
	if(Status != HE_OK) return error("Failed HSB close.", Status);
	free(frame1);
	free(frame2);
	runstats=0;

	return(0);
}


void CALLBACK Statsfunc(HWND hwnd, UINT uint, UINT uint2, ULONG longd)
{
	    HE_BYTE PCIFrames, PipelineFrames;
		HE_DWORD camera_frames,temp;

	if (runstats)
	{
			PCIFrames=goodframes/10; /*because timer is set to 10 seconds */
			readUserData(slotnumber,&PipelineFrames,1,READ_PIPELINE_STATS_LOW);
			camera_frames=PipelineFrames;
			readUserData(slotnumber,&PipelineFrames,1,READ_PIPELINE_STATS_HI);
			temp=(HE_DWORD)PipelineFrames;
			camera_frames|=(temp<<8);
			camera_frames=camera_frames/10;
			sprintf(Status_Text,"PCI Frames: %d, Pipeline Frames: %d", PCIFrames,camera_frames);
			PostMessage(pparams->hwnd,FPGA_STATUS,0,0);
			goodframes=0;
	}

}



void setConvolutionConstants(char *devname, HE_WORD Board, int Slot, unsigned long command){
		if(command == vSOBEL){
			SendUserData(devname, Board, Slot, K_A1, 0x00);
			SendUserData(devname, Board, Slot, K_A2, 0x00);
			SendUserData(devname, Board, Slot, K_A3, 0x00);
			SendUserData(devname, Board, Slot, K_A4, 0x00);
			SendUserData(devname, Board, Slot, K_A5, 0x00);
			SendUserData(devname, Board, Slot, K_B1, 0x00);
			SendUserData(devname, Board, Slot, K_B2, 0x01);
			SendUserData(devname, Board, Slot, K_B3, 0x02);
			SendUserData(devname, Board, Slot, K_B4, 0x01);
			SendUserData(devname, Board, Slot, K_B5, 0x00);
			SendUserData(devname, Board, Slot, K_C1, 0x00);
			SendUserData(devname, Board, Slot, K_C2, 0x00);
			SendUserData(devname, Board, Slot, K_C3, 0x00);
			SendUserData(devname, Board, Slot, K_C4, 0x00);
			SendUserData(devname, Board, Slot, K_C5, 0x00);
			SendUserData(devname, Board, Slot, K_D1, 0x00);
			SendUserData(devname, Board, Slot, K_D2, 0xFF);
			SendUserData(devname, Board, Slot, K_D3, 0xFE);
			SendUserData(devname, Board, Slot, K_D4, 0xFF);
			SendUserData(devname, Board, Slot, K_D5, 0x00);
			SendUserData(devname, Board, Slot, K_E1, 0x00);
			SendUserData(devname, Board, Slot, K_E2, 0x00);
			SendUserData(devname, Board, Slot, K_E3, 0x00);
			SendUserData(devname, Board, Slot, K_E4, 0x00);
			SendUserData(devname, Board, Slot, K_E5, 0x00);
			SendUserData(devname, Board, Slot, SHIFT, 0x00);
		}else if(command == hSOBEL){
			SendUserData(devname, Board, Slot, K_A1, 0x00);
			SendUserData(devname, Board, Slot, K_A2, 0x00);
			SendUserData(devname, Board, Slot, K_A3, 0x00);
			SendUserData(devname, Board, Slot, K_A4, 0x00);
			SendUserData(devname, Board, Slot, K_A5, 0x00);
			SendUserData(devname, Board, Slot, K_B1, 0x00);
			SendUserData(devname, Board, Slot, K_B2, 0x01);
			SendUserData(devname, Board, Slot, K_B3, 0x00);
			SendUserData(devname, Board, Slot, K_B4, 0xFF);
			SendUserData(devname, Board, Slot, K_B5, 0x00);
			SendUserData(devname, Board, Slot, K_C1, 0x00);
			SendUserData(devname, Board, Slot, K_C2, 0x02);
			SendUserData(devname, Board, Slot, K_C3, 0x00);
			SendUserData(devname, Board, Slot, K_C4, 0xFE);
			SendUserData(devname, Board, Slot, K_C5, 0x00);
			SendUserData(devname, Board, Slot, K_D1, 0x00);
			SendUserData(devname, Board, Slot, K_D2, 0x01);
			SendUserData(devname, Board, Slot, K_D3, 0x00);
			SendUserData(devname, Board, Slot, K_D4, 0xFF);
			SendUserData(devname, Board, Slot, K_D5, 0x00);
			SendUserData(devname, Board, Slot, K_E1, 0x00);
			SendUserData(devname, Board, Slot, K_E2, 0x00);
			SendUserData(devname, Board, Slot, K_E3, 0x00);
			SendUserData(devname, Board, Slot, K_E4, 0x00);
			SendUserData(devname, Board, Slot, K_E5, 0x00);
			SendUserData(devname, Board, Slot, SHIFT, 0x00);
		}else if(command == vPREWITT){
			SendUserData(devname, Board, Slot, K_A1, 0x00);
			SendUserData(devname, Board, Slot, K_A2, 0x00);
			SendUserData(devname, Board, Slot, K_A3, 0x00);
			SendUserData(devname, Board, Slot, K_A4, 0x00);
			SendUserData(devname, Board, Slot, K_A5, 0x00);
			SendUserData(devname, Board, Slot, K_B1, 0x00);
			SendUserData(devname, Board, Slot, K_B2, 0x01);
			SendUserData(devname, Board, Slot, K_B3, 0x01);
			SendUserData(devname, Board, Slot, K_B4, 0x01);
			SendUserData(devname, Board, Slot, K_B5, 0x00);
			SendUserData(devname, Board, Slot, K_C1, 0x00);
			SendUserData(devname, Board, Slot, K_C2, 0x00);
			SendUserData(devname, Board, Slot, K_C3, 0x00);
			SendUserData(devname, Board, Slot, K_C4, 0x00);
			SendUserData(devname, Board, Slot, K_C5, 0x00);
			SendUserData(devname, Board, Slot, K_D1, 0x00);
			SendUserData(devname, Board, Slot, K_D2, 0xFF);
			SendUserData(devname, Board, Slot, K_D3, 0xFF);
			SendUserData(devname, Board, Slot, K_D4, 0xFF);
			SendUserData(devname, Board, Slot, K_D5, 0x00);
			SendUserData(devname, Board, Slot, K_E1, 0x00);
			SendUserData(devname, Board, Slot, K_E2, 0x00);
			SendUserData(devname, Board, Slot, K_E3, 0x00);
			SendUserData(devname, Board, Slot, K_E4, 0x00);
			SendUserData(devname, Board, Slot, K_E5, 0x00);
			SendUserData(devname, Board, Slot, SHIFT, 0x00);
		}else if(command == hPREWITT){
			SendUserData(devname, Board, Slot, K_A1, 0x00);
			SendUserData(devname, Board, Slot, K_A2, 0x00);
			SendUserData(devname, Board, Slot, K_A3, 0x00);
			SendUserData(devname, Board, Slot, K_A4, 0x00);
			SendUserData(devname, Board, Slot, K_A5, 0x00);
			SendUserData(devname, Board, Slot, K_B1, 0x00);
			SendUserData(devname, Board, Slot, K_B2, 0x01);
			SendUserData(devname, Board, Slot, K_B3, 0x00);
			SendUserData(devname, Board, Slot, K_B4, 0xFF);
			SendUserData(devname, Board, Slot, K_B5, 0x00);
			SendUserData(devname, Board, Slot, K_C1, 0x00);
			SendUserData(devname, Board, Slot, K_C2, 0x01);
			SendUserData(devname, Board, Slot, K_C3, 0x00);
			SendUserData(devname, Board, Slot, K_C4, 0xFF);
			SendUserData(devname, Board, Slot, K_C5, 0x00);
			SendUserData(devname, Board, Slot, K_D1, 0x00);
			SendUserData(devname, Board, Slot, K_D2, 0x01);
			SendUserData(devname, Board, Slot, K_D3, 0x00);
			SendUserData(devname, Board, Slot, K_D4, 0xFF);
			SendUserData(devname, Board, Slot, K_D5, 0x00);
			SendUserData(devname, Board, Slot, K_E1, 0x00);
			SendUserData(devname, Board, Slot, K_E2, 0x00);
			SendUserData(devname, Board, Slot, K_E3, 0x00);
			SendUserData(devname, Board, Slot, K_E4, 0x00);
			SendUserData(devname, Board, Slot, K_E5, 0x00);
			SendUserData(devname, Board, Slot, SHIFT, 0x00);
		}else if(command == LAPLACIAN){
			SendUserData(devname, Board, Slot, K_A1, 0x00);
			SendUserData(devname, Board, Slot, K_A2, 0x00);
			SendUserData(devname, Board, Slot, K_A3, 0x00);
			SendUserData(devname, Board, Slot, K_A4, 0x00);
			SendUserData(devname, Board, Slot, K_A5, 0x00);
			SendUserData(devname, Board, Slot, K_B1, 0x00);
			SendUserData(devname, Board, Slot, K_B2, 0xFF);
			SendUserData(devname, Board, Slot, K_B3, 0xFF);
			SendUserData(devname, Board, Slot, K_B4, 0xFF);
			SendUserData(devname, Board, Slot, K_B5, 0x00);
			SendUserData(devname, Board, Slot, K_C1, 0x00);
			SendUserData(devname, Board, Slot, K_C2, 0xFF);
			SendUserData(devname, Board, Slot, K_C3, 0x08);
			SendUserData(devname, Board, Slot, K_C4, 0xFF);
			SendUserData(devname, Board, Slot, K_C5, 0x00);
			SendUserData(devname, Board, Slot, K_D1, 0x00);
			SendUserData(devname, Board, Slot, K_D2, 0xFF);
			SendUserData(devname, Board, Slot, K_D3, 0xFF);
			SendUserData(devname, Board, Slot, K_D4, 0xFF);
			SendUserData(devname, Board, Slot, K_D5, 0x00);
			SendUserData(devname, Board, Slot, K_E1, 0x00);
			SendUserData(devname, Board, Slot, K_E2, 0x00);
			SendUserData(devname, Board, Slot, K_E3, 0x00);
			SendUserData(devname, Board, Slot, K_E4, 0x00);
			SendUserData(devname, Board, Slot, K_E5, 0x00);
			SendUserData(devname, Board, Slot, SHIFT, 0x00);
		}else if(command == LPFILTER){
			SendUserData(devname, Board, Slot, K_A1, 0x00);
			SendUserData(devname, Board, Slot, K_A2, 0x00);
			SendUserData(devname, Board, Slot, K_A3, 0x00);
			SendUserData(devname, Board, Slot, K_A4, 0x00);
			SendUserData(devname, Board, Slot, K_A5, 0x00);
			SendUserData(devname, Board, Slot, K_B1, 0x00);
			SendUserData(devname, Board, Slot, K_B2, 0x01);
			SendUserData(devname, Board, Slot, K_B3, 0x02);
			SendUserData(devname, Board, Slot, K_B4, 0x01);
			SendUserData(devname, Board, Slot, K_B5, 0x00);
			SendUserData(devname, Board, Slot, K_C1, 0x00);
			SendUserData(devname, Board, Slot, K_C2, 0x02);
			SendUserData(devname, Board, Slot, K_C3, 0x04);
			SendUserData(devname, Board, Slot, K_C4, 0x02);
			SendUserData(devname, Board, Slot, K_C5, 0x00);
			SendUserData(devname, Board, Slot, K_D1, 0x00);
			SendUserData(devname, Board, Slot, K_D2, 0x01);
			SendUserData(devname, Board, Slot, K_D3, 0x02);
			SendUserData(devname, Board, Slot, K_D4, 0x01);
			SendUserData(devname, Board, Slot, K_D5, 0x00);
			SendUserData(devname, Board, Slot, K_E1, 0x00);
			SendUserData(devname, Board, Slot, K_E2, 0x00);
			SendUserData(devname, Board, Slot, K_E3, 0x00);
			SendUserData(devname, Board, Slot, K_E4, 0x00);
			SendUserData(devname, Board, Slot, K_E5, 0x00);
			SendUserData(devname, Board, Slot, SHIFT, 0x0C);
		}else if(command == SHARPEN){
			SendUserData(devname, Board, Slot, K_A1, 0x00);
			SendUserData(devname, Board, Slot, K_A2, 0x00);
			SendUserData(devname, Board, Slot, K_A3, 0x00);
			SendUserData(devname, Board, Slot, K_A4, 0x00);
			SendUserData(devname, Board, Slot, K_A5, 0x00);
			SendUserData(devname, Board, Slot, K_B1, 0x00);
			SendUserData(devname, Board, Slot, K_B2, 0xFF);
			SendUserData(devname, Board, Slot, K_B3, 0xFF);
			SendUserData(devname, Board, Slot, K_B4, 0xFF);
			SendUserData(devname, Board, Slot, K_B5, 0x00);
			SendUserData(devname, Board, Slot, K_C1, 0x00);
			SendUserData(devname, Board, Slot, K_C2, 0xFF);
			SendUserData(devname, Board, Slot, K_C3, 0x10);
			SendUserData(devname, Board, Slot, K_C4, 0xFF);
			SendUserData(devname, Board, Slot, K_C5, 0x00);
			SendUserData(devname, Board, Slot, K_D1, 0x00);
			SendUserData(devname, Board, Slot, K_D2, 0xFF);
			SendUserData(devname, Board, Slot, K_D3, 0xFF);
			SendUserData(devname, Board, Slot, K_D4, 0xFF);
			SendUserData(devname, Board, Slot, K_D5, 0x00);
			SendUserData(devname, Board, Slot, K_E1, 0x00);
			SendUserData(devname, Board, Slot, K_E2, 0x00);
			SendUserData(devname, Board, Slot, K_E3, 0x00);
			SendUserData(devname, Board, Slot, K_E4, 0x00);
			SendUserData(devname, Board, Slot, K_E5, 0x00);
			SendUserData(devname, Board, Slot, SHIFT, 0x0D);
		}else if(command == vSOBEL5X5){
			SendUserData(devname, Board, Slot, K_A1, 0x01);
			SendUserData(devname, Board, Slot, K_A2, 0x02);
			SendUserData(devname, Board, Slot, K_A3, 0x03);
			SendUserData(devname, Board, Slot, K_A4, 0x02);
			SendUserData(devname, Board, Slot, K_A5, 0x01);
			SendUserData(devname, Board, Slot, K_B1, 0x02);
			SendUserData(devname, Board, Slot, K_B2, 0x03);
			SendUserData(devname, Board, Slot, K_B3, 0x04);
			SendUserData(devname, Board, Slot, K_B4, 0x03);
			SendUserData(devname, Board, Slot, K_B5, 0x02);
			SendUserData(devname, Board, Slot, K_C1, 0x00);
			SendUserData(devname, Board, Slot, K_C2, 0x00);
			SendUserData(devname, Board, Slot, K_C3, 0x00);
			SendUserData(devname, Board, Slot, K_C4, 0x00);
			SendUserData(devname, Board, Slot, K_C5, 0x00);
			SendUserData(devname, Board, Slot, K_D1, 0xFE);
			SendUserData(devname, Board, Slot, K_D2, 0xFD);
			SendUserData(devname, Board, Slot, K_D3, 0xFC);
			SendUserData(devname, Board, Slot, K_D4, 0xFD);
			SendUserData(devname, Board, Slot, K_D5, 0xFE);
			SendUserData(devname, Board, Slot, K_E1, 0xFF);
			SendUserData(devname, Board, Slot, K_E2, 0xFE);
			SendUserData(devname, Board, Slot, K_E3, 0xFD);
			SendUserData(devname, Board, Slot, K_E4, 0xFE);
			SendUserData(devname, Board, Slot, K_E5, 0xFF);
			SendUserData(devname, Board, Slot, SHIFT, 0x00);
		}else if(command == hSOBEL5X5){
			SendUserData(devname, Board, Slot, K_A1, 0x01);
			SendUserData(devname, Board, Slot, K_A2, 0x02);
			SendUserData(devname, Board, Slot, K_A3, 0x00);
			SendUserData(devname, Board, Slot, K_A4, 0xFE);
			SendUserData(devname, Board, Slot, K_A5, 0xFF);
			SendUserData(devname, Board, Slot, K_B1, 0x02);
			SendUserData(devname, Board, Slot, K_B2, 0x03);
			SendUserData(devname, Board, Slot, K_B3, 0x00);
			SendUserData(devname, Board, Slot, K_B4, 0xFD);
			SendUserData(devname, Board, Slot, K_B5, 0xFE);
			SendUserData(devname, Board, Slot, K_C1, 0x03);
			SendUserData(devname, Board, Slot, K_C2, 0x04);
			SendUserData(devname, Board, Slot, K_C3, 0x00);
			SendUserData(devname, Board, Slot, K_C4, 0xFC);
			SendUserData(devname, Board, Slot, K_C5, 0xFD);
			SendUserData(devname, Board, Slot, K_D1, 0x02);
			SendUserData(devname, Board, Slot, K_D2, 0x03);
			SendUserData(devname, Board, Slot, K_D3, 0x00);
			SendUserData(devname, Board, Slot, K_D4, 0xFD);
			SendUserData(devname, Board, Slot, K_D5, 0xFE);
			SendUserData(devname, Board, Slot, K_E1, 0x01);
			SendUserData(devname, Board, Slot, K_E2, 0x02);
			SendUserData(devname, Board, Slot, K_E3, 0x00);
			SendUserData(devname, Board, Slot, K_E4, 0xFE);
			SendUserData(devname, Board, Slot, K_E5, 0xFF);
			SendUserData(devname, Board, Slot, SHIFT, 0x00);
		}else if(command == LAPLACIAN5X5){
			SendUserData(devname, Board, Slot, K_A1, 0xFF);
			SendUserData(devname, Board, Slot, K_A2, 0xFD);
			SendUserData(devname, Board, Slot, K_A3, 0xFC);
			SendUserData(devname, Board, Slot, K_A4, 0xFD);
			SendUserData(devname, Board, Slot, K_A5, 0xFF);
			SendUserData(devname, Board, Slot, K_B1, 0xFD);
			SendUserData(devname, Board, Slot, K_B2, 0x00);
			SendUserData(devname, Board, Slot, K_B3, 0x06);
			SendUserData(devname, Board, Slot, K_B4, 0x00);
			SendUserData(devname, Board, Slot, K_B5, 0xFD);
			SendUserData(devname, Board, Slot, K_C1, 0xFC);
			SendUserData(devname, Board, Slot, K_C2, 0x06);
			SendUserData(devname, Board, Slot, K_C3, 0x14);
			SendUserData(devname, Board, Slot, K_C4, 0x06);
			SendUserData(devname, Board, Slot, K_C5, 0xFC);
			SendUserData(devname, Board, Slot, K_D1, 0xFD);
			SendUserData(devname, Board, Slot, K_D2, 0x00);
			SendUserData(devname, Board, Slot, K_D3, 0x06);
			SendUserData(devname, Board, Slot, K_D4, 0x00);
			SendUserData(devname, Board, Slot, K_D5, 0xFD);
			SendUserData(devname, Board, Slot, K_E1, 0xFF);
			SendUserData(devname, Board, Slot, K_E2, 0xFD);
			SendUserData(devname, Board, Slot, K_E3, 0xFC);
			SendUserData(devname, Board, Slot, K_E4, 0xFD);
			SendUserData(devname, Board, Slot, K_E5, 0xFF);
			SendUserData(devname, Board, Slot, SHIFT, 0x00);
	}
}



/****************** HSB setup *******************************/
int hsb_setup(char *devname, HE_WORD Board, HE_BYTE Slot,CAMERA *d)
{
int camera_control;
int pixel_stop;
int lpixel_start,lpixel_stop;
int hpixel_start,hpixel_stop;
int line_start;
int line_stop;
int lline_start,lline_stop;
int hline_start,hline_stop;
int pixels_per_line;    /* Used to store the number of pixels in a line */
int lines_per_frame;    /* Used to store the number of lines in a frame */
int pixel_size;
int pixel_size_to_set;
int mode;
int colourPicker;
int mode_reg_value;
unsigned char data;
HE_BYTE addr;
HE_BYTE reply[10];


#define AUTO_ROI_RESET 0x80


camera_control  =  (d->cam_control  & 0x0F);
camera_control |= ((d->cam_use_dval & 0x01) << 4);
camera_control |= ((d->cam_polarity & 0x01) << 5);

/* tell the auto ROI to scan the camera image so that we can read that information*/
addr = CAM_CONTROL_ADDR; /* Camera Control register address */
data = camera_control | AUTO_ROI_RESET;
SendUserData(devname, Board, Slot, addr, data);

/* de-assert the ROI RESET */ 
data = camera_control;
SendUserData(devname, Board, Slot, addr, data);

/* now we have to wait until it has scanned */
do {
	Sleep(10);
	readUserData(Slot,reply,10,AUTO_ROI_VALID_ADDR);
	reply[0] = reply[0]>>7;
} while (reply[0] == 0);


/* Calculate the correct frame size */	
readUserData(Slot,reply,10,PIXEL_START_LOW_ADDR);
lpixel_start = reply[0];

readUserData(Slot,reply,10,PIXEL_START_HIGH_ADDR);
hpixel_start = reply[0] & 0x0f;

readUserData(Slot,reply,10,PIXEL_STOP_LOW_ADDR);
lpixel_stop = reply[0];

readUserData(Slot,reply,10,PIXEL_STOP_HIGH_ADDR);
hpixel_stop = reply[0] & 0x0f;

pixels_per_line = (((hpixel_stop<<8)|lpixel_stop)-((hpixel_start<<8)|lpixel_start));

if (pixels_per_line > IMAGE_X) pixels_per_line=IMAGE_X;
else error("requested pixels more than pixels available ",pixels_per_line);

pixel_stop = pixels_per_line+((hpixel_start<<8)|lpixel_start) -1;
lpixel_stop =  pixel_stop     & 0xff;
hpixel_stop = (pixel_stop>>8) & 0x0f;

readUserData(Slot,reply,10,LINE_START_LOW_ADDR);
lline_start = reply[0];

readUserData(Slot,reply,10,LINE_START_HIGH_ADDR);
hline_start = reply[0] & 0x0f;

pixel_size = reply[0]>>4;
pixel_size = pixel_size & 0x0f;

if (pixel_size >= 8) /* 16 bit pixels */
{
	error("this demo cannot support a camera with 16 bit pixels",0);
	exit(0);
}
else if (pixel_size >= 4) /* 14 bit pixels*/
{
	pixel_size_to_set = 3;
//	error("14 bit pixels",0);
}
else if (pixel_size >= 2) /* 12 pixels*/
{
	pixel_size_to_set = 2;
//	error("12 bit pixels",0);
}
else if (pixel_size >= 1) /* 10 bit pixels*/
{
	pixel_size_to_set = 1;
//	error("10 bit pixels",0);
}
else
{
	pixel_size_to_set = 0;
}

readUserData(Slot,reply,10,LINE_STOP_LOW_ADDR);
lline_stop = reply[0];

readUserData(Slot,reply,10,LINE_STOP_HIGH_ADDR);
hline_stop = reply[0] & 0x0f;

lines_per_frame = (((hline_stop<<8)|lline_stop)-((hline_start<<8)|lline_start));

if (lines_per_frame > IMAGE_Y) lines_per_frame = IMAGE_Y;
else error("requested lines more than lines available ",lines_per_frame);

/* Check if first line is line 0, if so shift to line 1 */
/* This is because some cameras produce incomplete first lines
/* and so we should begin capture without line 0 */
line_start = (hline_start << 8) | lline_start;
if (line_start == 0) {
	lline_start = 1;
	hline_start = 0;
}

line_stop = lines_per_frame+((hline_start<<8)|lline_start) - 1;
lline_stop =  line_stop     & 0xff;
hline_stop = (line_stop>>8) & 0x0f;

// MODE
// Set to 4 for 4x8,2 for 2x16, 1 for 1x32
mode = 4;

// For colour cameras with Bayer outputs
colourPicker = CAM_COLOUR_PICKER;

mode_reg_value = mode | (pixel_size_to_set << 4) | (colourPicker << 7);

SendUserData(devname, Board, Slot, MODE_REG_ADDR, mode_reg_value);

SendUserData(devname, Board, Slot, PIXEL_START_LOW_ADDR, lpixel_start);

SendUserData(devname, Board, Slot, PIXEL_START_HIGH_ADDR, hpixel_start);

SendUserData(devname, Board, Slot, PIXEL_STOP_LOW_ADDR, lpixel_stop);

SendUserData(devname, Board, Slot, PIXEL_STOP_HIGH_ADDR, hpixel_stop);

SendUserData(devname, Board, Slot, LINE_START_LOW_ADDR, lline_start);

SendUserData(devname, Board, Slot, LINE_START_HIGH_ADDR, hline_start);

SendUserData(devname, Board, Slot, LINE_STOP_LOW_ADDR, lline_stop);

SendUserData(devname, Board, Slot, LINE_STOP_HIGH_ADDR, hline_stop);

return(0);
}



/************************************************************/

int error(char *string, int Status)
{
	char msg[1024];

		sprintf(msg, "%s (Error %x)", string, Status);
		MessageBox(NULL, msg, "Imaging Demo", MB_OK);
	
	Status = HeClose(&hDevice);
	return 0;
}



/************************************************************/
//                                                          //
//       Send User Data - Sends data over HSB               //
//                                                          //
/************************************************************/
void SendUserData(char *devname, HE_WORD Board, HE_BYTE slot, HE_BYTE Address, int Data)
{
HE_BYTE user[2];


	// Send user data
	user[0]=(HE_BYTE) Address;
	user[1]=(HE_BYTE) Data;

	Status = HeHSBSendMessage(hsbDevice, USER_WRITE, slot, &user[0], 2, 1000);
	if(Status != HE_OK){
    	MessageBox(NULL, Status + ": HSB Send Error", "Display Demo", MB_OK);
		exit(-2);
	}

}



/********************************************************************************/
/*																				*/
/*	read user data								     						    */
/*																				*/
/*	                                                							*/
/*       																		*/
/*																				*/
/********************************************************************************/
HE_BYTE readUserData(HE_BYTE slot, HE_BYTE *reply, int buffer_size, HE_BYTE user_addr)
{
	HE_BYTE  message_type;
	HE_DWORD bytes_read;
	HE_BYTE  send_data[5];
	HE_DWORD Status;

	send_data[0]=user_addr;
	send_data[1]=1;


	Status = HeHSBSendMessage(hsbDevice, USER_READ, slot, send_data, 2, 10000);	
	Status = HeHSBReceiveMessage(hsbDevice, &message_type, slot, reply, buffer_size, &bytes_read, 10000);
	if (Status == HE_OK) 
	{
		if (message_type != USER_READ_REPLY) 
		{
			MessageBox(NULL, "Error: expected user read reply", "Display Demo", MB_OK);
			exit(0);
		}
		else 
		{

			return(reply[0]);
		}
	}
	MessageBox(NULL, "Error: bad status from HSBread", "Display Demo", MB_OK);
	exit(0);

}



/********************************************************************************/
/*																				*/
/*	Initialise_Host_HSB															*/
/*																				*/
/*	This function sets up the host HSB interface of each board in the system.	*/
/*																				*/
/********************************************************************************/
int Initialise_Host_HSB(char *dev_name, HE_WORD board)
{

	/* Check parameters */
	if ((board < 0) || (board > 15)) {
		printf("Error: board ID is out of range (0 to 15)\n");
		return(-1);
	}

	Status = HeOpen(dev_name, board, HSB, &hsbDevice);
	if (Status != HE_OK) return(1);

	return(0);
}



/********************************************************************************/
/*																				*/
/*	readback_user																*/
/*																				*/
/*	This function is used to read back from the FPGA							*/
/*  slot.																		*/
/*																				*/
/********************************************************************************/
int readback_user(HE_BYTE slot, HE_BYTE *reply, int buffer_size, HE_BYTE user_addr)
{
	HE_BYTE  message_type;
	HE_DWORD bytes_read;
	HE_BYTE  send_data[5];

	send_data[0]=user_addr;
	send_data[1]=1;

	Status = HeHSBSendMessage(hsbDevice, USER_READ, slot, send_data, 2, 1000);
	Status = HeHSBReceiveMessage(hsbDevice, &message_type, slot, reply, buffer_size, &bytes_read, 1000);
	if (Status == HE_OK) 
	{
		if (message_type != USER_READ_REPLY) 
		{
			MessageBox(NULL, "Error: expected user read reply", "Display Demo", MB_OK);
			printf("Error: expected user read reply\n");
			return(-1);
		}
		else 
		{

			return(reply[0]);
		}
	}

	return(Status);
}



/********************************************************************************/
/*																				*/
/*	Error																		*/
/*																				*/
/*	General Error Routine														*/
/*																				*/
/********************************************************************************/
void Error(HE_DWORD errorcode)
{
	char text[1024];

	MessageBox(NULL, (char*)errorcode, "Display Demo", MB_OK);
	printf("In error routine\n");
	HeErr2Text(Status, text);
	printf("Failed - %s, OsError = %ld", text, errorcode);
	Print_Win32_Error(errorcode);
	printf("\n");

//	Status = HeClose(&hDevice);
	exit(1);
}



/********************************************************************************/
/*																				*/
/*	Display_Help																*/
/*																				*/
/*	This function displays help information about this program.					*/
/*																				*/
/********************************************************************************/
void Display_Help()
{
	printf("Imaging with FPGA host demo program\n"); 	

	return;
}

/********************************************************************************/
/*																				*/
/*	Print_Win32_Error															*/
/*																				*/
/*	Win32 Error Function														*/
/*																				*/
/********************************************************************************/
void Print_Win32_Error(HE_DWORD errorcode)
{
#ifdef _WIN32			/* This level of error code is only on WIN32 */
						/* don't even compile it if not WIN32!*/
	LPVOID lpMsgBuf;

	if(errorcode != 0) {
		printf(" (%s)\n", strerror(errorcode));

		FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
			NULL,
			errorcode, 
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), /* Default language*/
			(LPTSTR) &lpMsgBuf,
			0,
			NULL);
		printf("Sys Error - %s", lpMsgBuf);
	}
#endif				/* end of WIN32 specific error codes*/
}
