-- ==============================================================
-- FILE : GD12.VHD
-- ==============================================================
--
-- This file provides example VHDL code for using your FPGA
-- module with a HEGD12.
--
-- Copy and Paste the lines of VHDL from this file into your
-- own project.
--
-- ==============================================================
--
--  Date        : 15/09/2004
--  Author      : R. Williams - HUNT ENGINEERING
--  Description : GD12 Example Code
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     R. Williams    15-09-04    First Written
--
-- ==============================================================


  -- The following code must appear in the VHDL architecture header:

  -- Data Enable Signal Declarations
  signal GD12_CH0_EN : std_logic;
  signal GD12_CH1_EN : std_logic;
  signal GD12_CH2_EN : std_logic;
  signal GD12_CH3_EN : std_logic;
  signal GD12_CH4_EN : std_logic;
  signal GD12_CH5_EN : std_logic;
  signal GD12_CH6_EN : std_logic;
  signal GD12_CH7_EN : std_logic;

  -- Signed Data Bus Signal Declaration
  signal GD12_SGN_DATA : std_logic_vector(15 downto 0);

  -- Unsigned Data Bus Signal Declaration
  signal GD12_UNS_DATA : std_logic_vector(15 downto 0);


  -- The following code must appear in the VHDL architecture body.

  -- PLEASE NOTE. All signals defined below must be used in the HERON
  -- Input FIFO Clock Domain. That is, all are clocked using the signal
  -- FCLK_G.

  -- For this example VHDL, HERON Input FIFO 0 is connected to the data
  -- stream sourced by the HEGD12. Data is continually read from the
  -- Input FIFO.

  -- The signals INFIFO_READ_REQ, INFIFO_DVALID and INFIFO0_D are used.
  -- These signals are found in the port list of the USER-AP entity in
  -- your project.
  INFIFO_READ_REQ <= "000001";

  -- To take data from HEGD12 Channel 0, drive the data enable of your
  -- first stage of processing with the signal GD12_CH0_EN.
  GD12_CH0_EN <= INFIFO_DVALID(0) AND (NOT INFIFO0_D(18)) AND (NOT INFIFO0_D(17)) AND (NOT INFIFO0_D(16));

  -- To take data from HEGD12 Channel 1, drive the data enable of your
  -- first stage of processing with the signal GD12_CH1_EN.
  GD12_CH1_EN <= INFIFO_DVALID(0) AND (NOT INFIFO0_D(18)) AND (NOT INFIFO0_D(17)) AND (    INFIFO0_D(16));

  -- To take data from HEGD12 Channel 2, drive the data enable of your
  -- first stage of processing with the signal GD12_CH2_EN.
  GD12_CH2_EN <= INFIFO_DVALID(0) AND (NOT INFIFO0_D(18)) AND (    INFIFO0_D(17)) AND (NOT INFIFO0_D(16));

  -- To take data from HEGD12 Channel 3, drive the data enable of your
  -- first stage of processing with the signal GD12_CH3_EN.
  GD12_CH3_EN <= INFIFO_DVALID(0) AND (NOT INFIFO0_D(18)) AND (    INFIFO0_D(17)) AND (    INFIFO0_D(16));

  -- To take data from HEGD12 Channel 4, drive the data enable of your
  -- first stage of processing with the signal GD12_CH4_EN.
  GD12_CH4_EN <= INFIFO_DVALID(0) AND (    INFIFO0_D(18)) AND (NOT INFIFO0_D(17)) AND (NOT INFIFO0_D(16));

  -- To take data from HEGD12 Channel 5, drive the data enable of your
  -- first stage of processing with the signal GD12_CH5_EN.
  GD12_CH5_EN <= INFIFO_DVALID(0) AND (    INFIFO0_D(18)) AND (NOT INFIFO0_D(17)) AND (    INFIFO0_D(16));

  -- To take data from HEGD12 Channel 6, drive the data enable of your
  -- first stage of processing with the signal GD12_CH6_EN.
  GD12_CH6_EN <= INFIFO_DVALID(0) AND (    INFIFO0_D(18)) AND (    INFIFO0_D(17)) AND (NOT INFIFO0_D(16));

  -- To take data from HEGD12 Channel 7, drive the data enable of your
  -- first stage of processing with the signal GD12_CH7_EN.
  GD12_CH7_EN <= INFIFO_DVALID(0) AND (    INFIFO0_D(18)) AND (    INFIFO0_D(17)) AND (    INFIFO0_D(16));

  -- The HEGD12 provides 16-bit two's complement data

  -- To use signed data, connect to the following 16-bit bus
  GD12_SGN_DATA <= INFIFO0_D(15 downto 0);

  -- To use unsigned data, connect to the following 16-bit bus
  GD12_UNS_DATA <= (NOT INFIFO0_D(15)) & INFIFO0_D(14 downto 0);
